%FINDING IN-, OUT- and PSEUDO-COMMUNITIES in a (possibly) 
%DIRECTED, WEIGHTED network.
%
%Please cite: 
%P. Landi and C. Piccardi, 
%Revisiting community analysis in directed networks: In-, out-, and pseudo-communities, 
%Physical Review E, 89, 012814, 2014.
%
%Copyright: 2014, Pietro Landi & Carlo Piccardi, Politecnico di Milano, Italy
%email carlo.piccardi@polimi.it
%
%Last updated: January 29th, 2014
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%INPUT: 
%The file 
%   A_{netname}.mat 
%must be in the working directory and must contain the following variables
%in Matlab binary format:
%   i) A : NxN weight matrix defining the network. 
%       A(i,j) is the weight of the link i->j.
%       If all the (nonzero) weights are 1, the network is actually UNWEIGHTED.
%       If A is symmetric, the network is actually UNDIRECTED.
%   ii) labels : (optional) 1xN cell vector of node labels (e.g., names)
%
%
%OUTPUT: 
%No output file is saved: the following results are available in the
%workspace:
%   i) (pseudo)community found starting from node "seed" (if any):
%
%      com{seed}, labels(com{seed}), com_size(seed), com_phi(seed)
%
%  ii) DISTINCT (pseudo)communities: same set as above, but duplications
%      are removed (and the information on the starting node is lost)
%
%      dist_com{i}, labels(dist_com{i}), dist_size(i), dist_phi(i)
%
% iii) eps-(pseudo)communities: the subset of the distinct
%      (pseudo)communities such that phi<eps (eps is user specified)
%
%      eps_com{i}, labels(eps_com{i}), eps_size(i), eps_phi(i)
%
%  iv) PRUNED eps-(pseudo)communities: the subset of eps-(pseudo)communities
%      survived after pruning with user specified similarity thershold nu
%
%      pruned_com{i}, labels(pruned_com{i}), pruned_size(i), pruned_phi(i)
%
%
%PARAMETERS: 
%Please set the parameter values in the section below.

clear all
close all
set(0,'Units','pixels') 
scn = get(0,'ScreenSize');

global P P_ xP xP_ x x_ st N

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%SETTING PARAMETERS

%%%%%name of the network: the file A_{netname}.mat will be loaded
%%%%%UNCOMMENT the name of the network to be loaded

%-----the following networks are analyzed in the paper
%-----(see reference above): the datafiles are available in the 
%-----distribution package 

% netname='wtn';
% netname='wtn61';
% netname='wtn75';
% netname='ownership';
% netname='airports';
% netname='leadership';
% netname='prison';
% netname='littlerock';
% netname='stmarks';
% netname='stmartin';
% netname='ythan';
% netname='neural';
% netname='ecoli';
% netname='scerevisiae';
% netname='politicalblogs';
% netname='japanese';
% netname='netscience';
netname='ppi';

%%%%% which type of STRUCTURE should be sought for
%%%%%UNCOMMENT the line of the type of structure to be sought for
% st='oc'; st_string='OUT-COMMUNITIES'; phi_line='norm([1-alpha,1-beta,beta_],inf)';
% st='ic'; st_string='IN-COMMUNITIES'; phi_line='norm([1-alpha_,beta,1-beta_],inf)';
% st='ioc'; st_string='IN/OUT-COMMUNITIES'; phi_line='norm([1-alpha,1-alpha_,1-beta,1-beta_],inf)';
% st='op'; st_string='OUT-PSEUDOCOMMUNITIES'; phi_line='norm([alpha,1-beta,beta_],inf)';
% st='ip'; st_string='IN-PSEUDOCOMMUNITIES'; phi_line='norm([alpha_,beta,1-beta_],inf)';
st='iop'; st_string='IN/OUT-PSEUDOCOMMUNITIES'; phi_line='norm([alpha,alpha_,1-beta,1-beta_],inf)';
% st='ipoc'; st_string='IN-PSEUDOCOMMUNITIES/OUT-COMMUNITIES'; phi_line='norm([1-alpha,alpha_,1-beta,1-beta_],inf)';
% st='icop'; st_string='IN-COMMUNITIES/OUT-PSEUDOCOMMUNITIES'; phi_line='norm([alpha,1-alpha_,1-beta,1-beta_],inf)';

%%%%% TELEPORTING PROBABILITY
% gamma=1; %set gamma=1 if the network is strongly connected
gamma=.85; %set gamma=0.85 otherwise

%%%%% STOPPING PARAMETER (see Supplemental Material of the above cited paper)
%%%%% the growth of the local community stops if the phi-minimum has depth larger than r
%%%%% default value: r=0.0
r=0.0;

%%%%%% MAXIMUM ALLOWABLE (PSEUDO)COMMUNITY SIZE (see Supplemental Material of the above cited paper)
%%%%%% the growth of the local community stops when larger or equal than n_max=round(N/ncom_ratio)
%%%%%% default value: ncom_ratio=1, so that n_max=N
ncom_ratio=1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%LOADING DATA, AND COMPUTING BASIC STATISTICS

disp([' '])
disp(['IN-, OUT-, PSEUDO-COMMUNITY ANALYSIS'])

%loads the NxN network matrix A 
%and (optionally) a Nx1 cell "labels" containing label strings
load(strcat('A_',netname,'.mat'));
A=full(A);

%if labels do not exist in the uploaded file,
%creates fictitious labels which are simply the node numbers
if length(find(char(who('*'))=='b'))==0 %labels do not exists in the file uploaded
    labels=cell(length(A),1);
    for i=1:length(A)
        labels(i)=cellstr(num2str(i));
    end;
end;

disp(['Network: ',netname,' - ',int2str(length(A)),' nodes'])

if length(find(diag(A)))>0
    disp(['WARNING: self-loops existing, set to zero.'])
    for i=1:length(A)
        A(i,i)=0;
    end;
end;       

%removing isolated nodes
disp(['Checking isolated nodes...'])
count=0;
lA=length(A);
for i=1:lA
    if ((sum(A(i-count,:))==0) && (sum(A(:,i-count))==0))
        A(i-count,:)=[];
        A(:,i-count)=[];
        labels(i-count)=[];
        count=count+1;
    end
end
if count>0
    disp(['WARNING: ',int2str(count),' isolated nodes removed.'])
    disp(['Network reduced to N = ',int2str(length(A)),' nodes.'])
    disp(['Reduced network saved in A_',netname,'_red.mat'])
    savename=strcat('A_',netname,'_red.mat');
    save(savename,'A','labels')
end;
%WARNING!!! From here on, the variables A and "labels" might not be the 
%original ones loaded with A_{netname}.mat !!!

if (max(max(A-A'))==0) && (min(min(A-A'))==0)
    disp('The network is UNDIRECTED.')
else
    disp('The network is DIRECTED.')
end

Abin=double(A>0); %binary adjacency matrix
Asim=min(1,(Abin+Abin')); %binary symmetrized (undirected) matrix
if (max(max(A-Abin))==0) && (min(min(A-Abin))==0)
    disp('The network is BINARY (unweighed).')
else
    disp('The network is WEIGHTED.')
end

k_in=sum(A); %row vector of node in-weights (or in-degrees)
k_out=sum(A')'; %column vector of node out-weights (or out-degrees)
m=sum(k_in); %total weight (or total number of links) in the network
N=length(k_in); %number of nodes

disp(['Computing the Markov matrices and stationary probability distributions...'])

%creating the Markov matrix P by row-normalizing A, and PPP for "unrecorded link teleportation"
P=zeros(N,N);
PPP=zeros(N,N);
v=zeros(1,N); %preference vector
rowsum=zeros(1,N);
for i=1:N
    rowsum(i)=sum(A(i,1:N));
end;
for i=1:N
    v=rowsum;
    v(i)=0; %to avoid creating self-loops
    v=v/sum(v);
    if rowsum(i)>0
        P(i,:)=A(i,:)/rowsum(i);
        PPP(i,:)=gamma*P(i,:)+(1-gamma)*v;
    else
        P(i,:)=v;
        PPP(i,:)=v;
    end;
end;

%computing Markov asymptotic distribution (x)
AAA=eye(N)-PPP'; 
AAA(N,:)=1;
bbb=zeros(N,1); 
bbb(N)=1;
x=AAA\bbb;
x=P'*x;x=x/sum(x); %unrecorded teleportation

xP=diag(x)*P; %asymptotic distribution times Markov matrix

A_=A'; %transpose matrix
%creating the Markov matrix P_ by row-normalizing A_, and PPP_ for "unrecorded link teleportation"
P_=zeros(N,N);
PPP_=zeros(N,N);
v_=zeros(1,N); %preference vector
rowsum_=zeros(1,N);
for i=1:N
    rowsum_(i)=sum(A_(i,1:N));
end;
for i=1:N
    v_=rowsum_;
    v_(i)=0; %to avoid creating self-loops
    v_=v_/sum(v_);
    if rowsum_(i)>0
        P_(i,:)=A_(i,:)/rowsum_(i);
        PPP_(i,:)=gamma*P_(i,:)+(1-gamma)*v_;
    else
        P_(i,:)=v_;
        PPP_(i,:)=v_;
    end;
end;

%computing Markov asymptotic distribution (x)
AAA=eye(N)-PPP_'; 
AAA(N,:)=1;
bbb=zeros(N,1); 
bbb(N)=1;
x_=AAA\bbb;
x_=P_'*x_;x_=x_/sum(x_); %unrecorded teleportation

xP_=diag(x_)*P_; %asymptotic distribution times Markov matrix

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%FINDING LOCAL COMMUNITIES

disp(['Growing the local (pseudo)community from each node...'])

%"listseed" is the list of nodes the local (pseudo)communities must be grown
%from; by default it contains all nodes...
listseed=1:N;
% ... but it can be modified in content and/or order.
% E.g., here I use (in order) the "kappa" nodes with largest total strength
%     kappa=10; %if kappa=N we are only changing the node order
%     [Bi,ix] = sort(-sum(A+A'));
%     Bi = Bi(1:kappa);
%     listseed = ix(1:kappa);
% E.g., here I use only a specific list of nodes (WARNING!!! Node numbering
% might have been modified when removing isolated nodes - see above)
%     listseed=[1 7 13 19];
%
% Anyway, "kappa" is the total number of nodes to be considered.
kappa=length(listseed);

max_ns=round(N/ncom_ratio); %maximum allowable (pseudo)community size
com=cell(N,1);              %com{i} is the structure found from seed=i
com_phi=zeros(N,1);         %com_phi(i) is the corresponding phi-value
com_size=zeros(N,1);        %com_size(i) is the corresponding size (n. of nodes)
set_vec=cell(N,1);

figure('OuterPosition',[scn(3)*.6 scn(4)*.1 scn(3)*.4 scn(4)*.45])
f1=get(0,'CurrentFigure');

count=0;
for seed=listseed
    
    count=count+1;    
    disp(['...node ',int2str(count),' of ',int2str(kappa),': ',char(labels(seed))])
    
    %growing a subnetwork from node 'seed'  
    current=[seed];
    sum_xP=0;
    sum_P=0;
    sum_x=x(seed);
    sum_xP_=0;
    sum_P_=0;
    sum_x_=x_(seed);
    neighbors=union(seed,union(find(P(seed,:)),find(P(:,seed))));
    phi_vec=zeros(1,N); %will contain the function phi(seed,m)
                        %see Fig. 2 in the Supplemental Material
    phi_vec(1)=1;
    
    best=[seed];                %best subnetwork to date
    phi_best=1;                 %best phi-value to date
    iii=2;
    found=0;
    
%   computing the function phi(seed,m)
    while found==0 && length(current)<max_ns
        
        %finds the "crown" of neighbors of the current subnetwork
        n_current=length(current);
        test=setdiff(neighbors,current);
        ntest=length(test);

        %disconnected component, no further neighbour: treated as a (pseudo)community
        if ntest==0 
            com{seed,1}=current;
            com_size(seed)=length(current);
            com_phi(seed)=phi(current);
            found=1;
            break
        end;

        phitest=zeros(1,N);
        for j=test
            alpha=(sum_xP+sum(xP(current,j))+sum(xP(j,current)))/(sum_x+x(j));
            alpha_=(sum_xP_+sum(xP_(current,j))+sum(xP_(j,current)))/(sum_x_+x_(j));
            beta=(sum_P+sum(P(current,j))+sum(P(j,current)))/(n_current+1);
            beta_=(sum_P_+sum(P_(current,j))+sum(P_(j,current)))/(n_current+1);

            phitest(j)=eval(phi_line);
        end;
        
        %getting the best (or least worse...) neighbor
        [ccc,jjj]=min(phitest(test));
        current=union(current,test(jjj));
        neighbors=union(neighbors,union(find(P(test(jjj),:)),find(P(:,test(jjj)))));
        
        sum_xP=(sum_xP+sum(xP(current,test(jjj)))+sum(xP(test(jjj),current)));
        sum_xP_=(sum_xP_+sum(xP_(current,test(jjj)))+sum(xP_(test(jjj),current)));
        sum_P=(sum_P+sum(P(current,test(jjj)))+sum(P(test(jjj),current)));
        sum_P_=(sum_P_+sum(P_(current,test(jjj)))+sum(P_(test(jjj),current)));
        sum_x=sum_x+x(test(jjj));
        sum_x_=sum_x_+x_(test(jjj));

        phi_vec(iii)=phi(current); 
        set_vec{iii,1}=current;
        
        if phi_vec(iii)<phi_best
            best=set_vec{iii,1};
            phi_best=phi_vec(iii);
        elseif phi_vec(iii)>phi_best+r
            %a subnetwork meeting stopping crierion (r) is found
            com{seed,1}=best;
            com_size(seed)=length(best);
            com_phi(seed)=phi_best;
            found=1;
        end;
        iii=iii+1;
        
    end;
        
%     %%%%%%%%%%%%%% REAL-TIME GRAPHICS: comment this block to speed-up
    figure(f1)
    if found==1
        plot(1:iii-1,phi_vec(1:iii-1),'x-',...
            [length(com{seed,1}) length(com{seed,1})],[0 com_phi(seed)],'ro-',...
            [length(com{seed,1}) length(com{seed,1})],[com_phi(seed) com_phi(seed)+r],'ro-')
    else
        plot(1:iii-1,phi_vec(1:iii-1),'x-',1:N,eps*ones(1,N),'k')
    end
    axis([1 max_ns 0 1])
    grid on
    title(['growing from node ',char(labels(seed))])
    xlabel('m'); ylabel('\phi')
    drawnow
%     pause
%     %%%%%%%%%%%%%%% end of REAL-TIME GRAPHICS %%%%%%%%%%%%%%%%%%%%%%%

end; %(pseudo)communities have been grown from all nodes


n_com=length(find(com_phi));   %number of structures found
disp([' '])
disp(['Found ',int2str(n_com),' ',st_string,' (possibly coincident)'])
disp(['Type "com{seed}" [or "labels(com{seed})"] to view the (pseudo)community found from node "seed" (if any),'])
disp(['and type "com_size(seed)" to view its size and "com_phi(seed)" to view its phi-value.'])

%finding distinct communities
nd_com=0;
if n_com>0
    disp([' '])
    disp(['Finding distinct (pseudo)communities...'])
    disp([' '])
    if n_com>0
        list_com=find(com_phi)';
        dist_com{1,1}=com{list_com(1)};     %distinct communities
        dist_phi(1)=com_phi(list_com(1));   %their phi-values
        dist_size(1)=com_size(list_com(1)); %their size
        nd_com=1;
        for i=list_com(2:n_com)
            coincident=0;
            j=1;
            %comparing com{i} with the current list dist_com
            while j<=nd_com && coincident==0
                if length(com{i})==length(dist_com{j}) && (sum(com{i}==dist_com{j})==length(com{i}))
                    coincident=1;
                end
                j=j+1;
            end
            if coincident==0    %add com{i} to the list of distinct communities
                nd_com=nd_com+1;
                dist_com{nd_com,1}=com{i};
                dist_phi(nd_com)=com_phi(i);
                dist_size(nd_com)=com_size(i);
            end;
        end
    end
    disp(['Found ',int2str(nd_com),' -DISTINCT- ',st_string,' : see Figure 2 for size/phi distribution.'])
    disp(['Type "dist_com{i}" [or "labels(dist_com{i})"] to view the i-th distinct (pseudo)community,'])
    disp(['and type "dist_size(i)" to view its size and "dist_phi(i)" to view its phi-value.'])
    
    figure('OuterPosition',[scn(3)*.6 scn(4)*.55 scn(3)*.4 scn(4)*.45])
    f2=get(0,'CurrentFigure');
    plot(dist_size,dist_phi,'xr','MarkerSize',10)
    axis([min(dist_size)-0.1*(max(dist_size)-min(dist_size)) max(dist_size)+0.1*(max(dist_size)-min(dist_size)) min(dist_phi)-0.1*(max(dist_phi)-min(dist_phi)) max(dist_phi)+0.1*(max(dist_phi)-min(dist_phi))])
    grid on
    title(['size/\phi distribution of distinct (pseudo)communities'])
    xlabel('size (n. of nodes)'); ylabel('\phi')
        
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% SELECTING AND PRUNING COMMUNITIES

while nd_com>0 %infinite loop, stopping only with ctrl-c
    disp([' '])
    disp(['Selecting eps-',st_string,' (those with phi<=eps).']) 
    eps=input('Set your desired eps value (or ctrl-c to stop):  eps = ');
  
    if exist('eps_com')==1
        clear eps_com eps_phi eps_size
    end
    
    %picking distinct communities if they have phi<=eps
    eps_n=0;
    for i=1:nd_com
        if dist_phi(i)<=eps
            eps_n=eps_n+1;
            eps_com{eps_n,1}=dist_com{i};
            eps_phi(eps_n)=dist_phi(i);
            eps_size(eps_n)=dist_size(i);
        end;
    end;
    
    disp([' '])
    disp(['Found ',int2str(eps_n),' eps-',st_string,' with eps = ',num2str(eps)])
    disp(['Type "eps_com{i}" [or "labels(eps_com{i})"] to view the i-th eps-(pseudo)community,'])
    disp(['and type "eps_size(i)" to view its size and "eps_phi(i)" to view its phi-value.'])

    %PRUNING eps-COMMUNITIES
    if eps_n>1

        disp([' '])
        nu=input('Set your desired value of pruning similarity threshold, nu = ');
        disp(['Recursive pruning with similarity threshold = ',num2str(nu),' ...'])
        
        old_com=eps_com;
        old_phi=eps_phi;
        old_n=length(eps_phi);
        pruning=1;
        while pruning==1
            disp(['... current number of (pseudo)communities = ',int2str(old_n)])
            pruning=0;
            MMM=zeros(old_n); %matrix of similarities among communities
            for i=1:old_n
                for j=i+1:old_n
                    MMM(i,j)=(length(intersect(old_com{i},old_com{j}))/length(union(old_com{i},old_com{j})))>nu;
                end
            end
            MMM=MMM+MMM';
            %now the maximal cliques of MMM are found...
            [MC]=maximalCliques(MMM);
            [nr,nc]=size(MC);   %nc is the number of maximal cliques
            %...and then keeping only the best (min phi) in each clique
            new_n=0;
            new_com=cell(old_n,1);
            new_phi=zeros(old_n,1);
            new_size=zeros(old_n,1);
            taken=zeros(old_n,1);
            for j=1:nc
                if sum(MC(:,j))>1 %not a trivial (one-cluster) clique
                    m_com=[];
                    temp=find(MC(:,j));
                    [m_phi,in]=min(old_phi(temp));
                    index=temp(in);
                    if taken(index)==0
                        m_com=old_com{index};
                        pruning=1;
                        new_n=new_n+1;
                        new_com{new_n,1}=m_com;
                        new_phi(new_n)=m_phi;
                        new_size(new_n)=length(m_com);
                    end
                    taken=taken+MC(:,j);
                end
            end
            %copying the (pseudo)communities not pruned
            for i=1:old_n
                if taken(i)==0
                    new_n=new_n+1;
                    new_com{new_n,1}=old_com{i};
                    new_phi(new_n)=old_phi(i);
                    new_size(new_n)=length(new_com{new_n,1});
                end;
            end
            old_com=new_com;
            old_phi=new_phi;
            old_n=new_n;
        end
        pruned_com=new_com;
        pruned_phi=new_phi;
        pruned_size=new_size;
        pruned_n=new_n;
        disp(['Found ',int2str(pruned_n),' -PRUNED- eps-',st_string,' with eps = ',num2str(eps)])
        disp(['Type "pruned_com{i}" [or "labels(pruned_com{i})"] to view the i-th pruned (pseudo)community,'])
        disp(['and type "pruned_size(i)" to view its size and "pruned_phi(i)" to view its phi-value.'])
        disp([' '])
        disp(['See Figure 1 for size/phi distribution of DISTINCT (red crosses) and PRUNED (blue circles) eps-',st_string])
            
        figure(f1)
        plot(eps_size,eps_phi,'xr',pruned_size,pruned_phi,'bo','MarkerSize',10)
        axis([min(dist_size)-0.1*(max(dist_size)-min(dist_size)) max(dist_size)+0.1*(max(dist_size)-min(dist_size)) min(eps_phi)-0.1*(max(eps_phi)-min(eps_phi)) max(eps_phi)+0.1*(max(eps_phi)-min(eps_phi))])
        grid on
        title(['size/\phi distribution of DISTINCT and PRUNED eps-(pseudo)communities'])
        xlabel('size (n. of nodes)'); ylabel('\phi')

        %comparing the set of eps-(pseudo)communities to the set of pruned ones
        pairs_bench=zeros(N);
        for m=1:eps_n
           for i=1:eps_size(m)
              for j=i+1:eps_size(m)
                  pairs_bench(eps_com{m}(i),eps_com{m}(j))=1;
              end
           end
        end

        pairs=zeros(N);
        for m=1:pruned_n
           for i=1:pruned_size(m)
              for j=i+1:pruned_size(m)
                  pairs(pruned_com{m}(i),pruned_com{m}(j))=1;
              end
           end
        end

        msum=pairs_bench+pairs;
        matchings=length(find(msum==2));
        recall=matchings/sum(sum(pairs_bench));
          
        disp([' '])
        disp(['Fraction of node pairs coupled by eps-',st_string])
        disp(['that are also coupled by pruned ones = ',num2str(recall)])

    end
    
end
